###############################################################################
###############################################################################
##                                Opciones.tcl
###############################################################################
###############################################################################
## Creates a dialog in which to choose the options for the program
###############################################################################
###############################################################################
## (c) 2001-2008 Andrs Garca Garca. fandom@telefonica.net
## You may distribute the contents of this file under the terms of the GPL v2
###############################################################################
###############################################################################

namespace eval Opciones {

###############################################################################
# UpdateSpinBox
#    Activates o deactivates the spinboxes.
#
# Parameter:
#    which: The spinbox to update
###############################################################################
proc UpdateSpinBox {which} {
    variable window
    variable optionsTemp

    if {$which=="internal"} {
        if {$optionsTemp(limitLevels)==1} {
            $window(intSpin) configure -state normal
            if {$optionsTemp(levels)==0} {
                set optionsTemp(levels) 1
            }
        } else {
            $window(intSpin) configure -state disable
            set optionsTemp(levels) 0
        }
    } else {
        if {$optionsTemp(external)==1} {
            $window(extSpin) configure -state normal
            if {$optionsTemp(exLevels)==0} {
                set optionsTemp(exLevels) 1
            }
        } else {
            $window(extSpin) configure -state disable
            set optionsTemp(exLevels) 0
        }
    }
    return
}

###############################################################################
# SetDefaultOptions
#    This procedures set the downloading options to the default values.
#
# Parameter:
#    The name of the options to set, it can be 'downOptions' or
#    'downOptionsTemp'
###############################################################################
proc SetDefaultOptions {optionsToSet} {
    global defaultDownOptions

    upvar #0 $optionsToSet options
    array set options [array get defaultDownOptions]

    if {$defaultDownOptions(levels)==-1} {
        set options(limitLevels) 0
    } else {
        set options(limitLevels) 1
    }
    return
}

###############################################################################
# SaveDefaultOptions
#    Sets the current values of the options in the dialog as default.
#############################################################################
proc SaveDefaultOptions {} {
    global defaultDownOptions
    variable optionsTemp

    array set defaultDownOptions [array get optionsTemp]
    if {$optionsTemp(limitLevels)==0} {
        set defaultDownOptions(levels) -1
    }
    return
}

###############################################################################
# ActivateMap
#    It is invoked when the user clicks in the 'Get map' checkbutton in the
#    options dialog.
#
# Side effects:
#    It disables or activates the 'Only Html checkbutton' and the 'Choose
#    filter' button, sets optionsTemp(onlyHtml) to '1' to get the map and
#    to '0' not to get it.
###############################################################################
proc ActivateMap {} {
    variable optionsTemp
    variable window

    if {$optionsTemp(map)==1} {
        $window(onlyHtml)  configure -state disable
        $window(chooseFil) configure -state disable
        set optionsTemp(onlyHtml) 1
    } else {
        $window(onlyHtml)  configure -state normal
        $window(chooseFil) configure -state normal
        set optionsTemp(onlyHtml) 0
    }

    return
}

###############################################################################
# ChangeFilter
#    Invokes the dialog to choose the filter for the download.
#
# Side effect:
#    Puts the returned filter into 'optionsTemp(filter)'
###############################################################################
proc ChangeFilter {} {
    variable window
    variable optionsTemp

    set optionsTemp(filter) [Herramientas::FilterFiles \
            $optionsTemp(filter) $window(win)]

    return
}

###############################################################################
# ChooseOptionsWindow
#    Creates the dialog in which to select the different options.
#
# Parameters:
#    parent: Path of the dialog over which this one will appear.
###############################################################################
proc ChooseOptionsWindow {parent} {
    global labelTitles labelButtons labelDialogs labelFrames getleftOptions
    global dirGetleft indexButtons indexDialogs
    variable optionsTemp
    variable done
    variable window

    set coord(x) [winfo rootx $parent]
    set coord(y) [winfo rooty $parent]

    if {[winfo exists .optionsWin]} {
        raise $window(win) .
        return
    }

    set win [toplevel .optionsWin]
    wm resizable $win 0 0
    wm title $win $labelTitles(optionsWin)
    wm geometry $win +[expr {$coord(x)+100}]+[expr {$coord(y)+100}]
    keynav::enableMnemonics  $win

    set extFrame  [ttk::frame $win.extFrame]
    set intFrame  [ttk::frame $extFrame.intFrame]
    set butFrame  [ttk::frame $extFrame.butFrame]

    set linkFrame [ttk::labelframe $intFrame.links -labelanchor nw             \
            -labelwidget [ttk::label $intFrame.linkslb                         \
            -textvariable labelFrames(links)]]
    raise $intFrame.linkslb

    set updirLinks  [ttk::checkbutton $linkFrame.updirLinks                    \
            -variable Opciones::optionsTemp(dir)                               \
            -textvariable labelDialogs(uplinks) -underline $indexDialogs(uplinks)]
    set updateLinks [ttk::checkbutton $linkFrame.updateLinks                   \
            -underline $indexDialogs(update)                                   \
            -variable Opciones::optionsTemp(update)                            \
            -textvariable labelDialogs(update)]
    set cgiLinks    [ttk::checkbutton $linkFrame.cgiLinks                      \
            -variable Opciones::optionsTemp(cgi)                               \
            -textvariable labelDialogs(cgi) -underline $indexDialogs(cgi)]

    set levelFrame [ttk::labelframe $intFrame.levels -labelanchor nw           \
            -labelwidget [ttk::label $intFrame.levelslb                        \
            -textvariable labelFrames(levels)]]
    raise $intFrame.levelslb
    set internal   [ttk::checkbutton $levelFrame.internal                      \
            -variable Opciones::optionsTemp(limitLevels)                       \
            -underline $indexDialogs(levels) -textvariable labelDialogs(levels)\
            -command "Opciones::UpdateSpinBox internal"]
    set external   [ttk::checkbutton $levelFrame.external                      \
            -variable Opciones::optionsTemp(external)                          \
            -textvariable labelDialogs(external)                               \
            -underline $indexDialogs(external)                                 \
            -command "Opciones::UpdateSpinBox external"]
    set intSpin [spinbox $levelFrame.intspin                                   \
            -from 0 -to 5 -width 3 -justify right                              \
            -background $getleftOptions(bg) -foreground $getleftOptions(fg)    \
            -textvariable Opciones::optionsTemp(levels)                        \
            -disabledbackground $getleftOptions(disBg)                         \
            -disabledforeground $getleftOptions(disFg)]
    set extSpin [spinbox $levelFrame.extspin                                   \
            -from 0 -to 3 -width 3 -justify right                              \
            -background $getleftOptions(bg) -foreground $getleftOptions(fg)    \
            -textvariable Opciones::optionsTemp(exLevels)                      \
            -disabledbackground $getleftOptions(disBg)                         \
            -disabledforeground $getleftOptions(disFg)]

    set imageFrame   [ttk::labelframe $intFrame.image   -labelanchor nw        \
            -labelwidget [ttk::label  $intFrame.imagelb                        \
            -textvariable labelFrames(images)]]
    raise $intFrame.imagelb
    set all          [ttk::radiobutton $imageFrame.all        -value 0         \
            -textvariable labelDialogs(allImages)                              \
            -underline $indexDialogs(allImages)                                \
            -variable Opciones::optionsTemp(images)]
    set onlyThumbs   [ttk::radiobutton $imageFrame.onlyThumbs -value 1         \
            -variable Opciones::optionsTemp(images)                            \
            -underline $indexDialogs(onlyThumb)                                \
            -textvariable labelDialogs(onlyThumb)]
    set noThumbs     [ttk::radiobutton $imageFrame.noThumbs   -value 2         \
            -variable Opciones::optionsTemp(images)                            \
            -textvariable labelDialogs(noThumb) -underline $indexDialogs(noThumb)]

    set filterFrame  [ttk::labelframe $intFrame.filters -labelanchor nw        \
            -labelwidget [ttk::label  $intFrame.filterslb                      \
            -textvariable labelFrames(filters)]]
    raise $intFrame.filterslb
    set mapLinks     [ttk::checkbutton $filterFrame.mapLinks                   \
	        -textvariable labelDialogs(getMap)                                 \
            -variable Opciones::optionsTemp(map) -underline $indexDialogs(map) \
            -command Opciones::ActivateMap -underline $indexDialogs(getMap)]
    set onlyHtml     [ttk::checkbutton $filterFrame.onlyHtml                   \
            -variable Opciones::optionsTemp(onlyHtml)                          \
            -underline $indexDialogs(onlyHtml) -textvariable labelDialogs(onlyHtml)]
    set chooseFilter [ttk::button $filterFrame.choose                          \
            -textvariable labelButtons(chooseFilter)                           \
            -underline $indexButtons(chooseFilter) -command Opciones::ChangeFilter]

    set accept  [ttk::button $butFrame.accept -width 8                         \
        -textvariable labelButtons(ok)                                         \
        -underline $indexButtons(ok)     -command "Opciones::ChooseOptionsControl"]
    set cancel  [ttk::button $butFrame.cancel -width 8                         \
        -textvariable labelButtons(cancel)                                     \
		-underline $indexButtons(cancel) -command "set Opciones::done 0"]
    set help    [ttk::button $butFrame.help -width 8                           \
        -textvariable  labelButtons(help)                                      \
        -underline $indexButtons(help)                                         \
        -command "Ayuda::Ayuda howtoDown#Options"]
    set saveOps [ttk::button $butFrame.save -width 8                           \
        -textvariable labelButtons(save)                                       \
        -underline $indexButtons(save)   -command "Opciones::SaveDefaultOptions"]
    set loadOps [ttk::button $butFrame.load -width 8                           \
        -textvariable labelButtons(load)                                       \
        -underline $indexButtons(load)                                         \
        -command "
                Opciones::SetDefaultOptions Opciones::optionsTemp
                Opciones::ActivateMap
                Opciones::UpdateSpinBox internal
                Opciones::UpdateSpinBox external
        "]

    set window(win)       $win
    set window(intSpin)   $intSpin
    set window(extSpin)   $extSpin
    set window(onlyHtml)  $onlyHtml
    set window(chooseFil) $chooseFilter

    if {$optionsTemp(map)==1} {
        ActivateMap
    }

    if {$::tcl_version>=8.5} {
        grid anchor $extFrame    c
        grid anchor $linkFrame   c
        grid anchor $levelFrame  c
        grid anchor $butFrame    c
        grid anchor $filterFrame c
        grid anchor $imageFrame  c
    }

    grid $extFrame -ipady 5 -ipadx 2
    grid $intFrame

    grid $linkFrame   $levelFrame $butFrame -ipadx 5 -sticky news
    grid $filterFrame $imageFrame           -ipadx 5 -sticky news

    grid configure $linkFrame   -padx {8 5}
    grid configure $filterFrame -padx {8 5}
    grid configure $butFrame    -rowspan 2

    grid $updirLinks        -sticky w 
    grid $updateLinks       -sticky w
    grid $cgiLinks          -sticky w 

    grid $internal $intSpin -sticky w
    grid $external $extSpin -sticky w

    grid $mapLinks          -sticky w
    grid $onlyHtml          -sticky w
    grid $chooseFilter

    grid $all               -sticky w
    grid $onlyThumbs        -sticky w
    grid $noThumbs          -sticky w

    grid $accept  -pady 3 -padx 0
    grid $cancel  -pady 3 -padx 0
    grid $loadOps -pady 3 -padx 0
    grid $saveOps -pady 3 -padx 0
    grid $help    -pady 3 -padx 0

    focus $win
    bind  $win <Escape> "set Opciones::done 0"

    return
}

###############################################################################
# ChooseOptionsControl
#    Makes sure that the options in the dialog make sense before closing.
###############################################################################
proc ChooseOptionsControl {} {
    global labelTitles labelMessages
    variable window
    variable done
    variable optionsTemp

    if {$optionsTemp(limitLevels)==1} {
        set intLimit $optionsTemp(levels)
        if {![string is integer $intLimit]} {
            Dialogos::Dialogo $window(win).error -title $labelTitles(error) \
                    -message $labelMessages(intNaN)  -type ok
            return
        }
        if {$intLimit<0} {
            set optionsTemp(levels) 0
        }
        if {$intLimit>5} {
            set optionsTemp(levels) 5
        }
    }

    if {$optionsTemp(external)!=0} {
        set extLimit $optionsTemp(exLevels)
        if {![string is integer $extLimit]} {
            Dialogos::Dialogo $window(win).error -title $labelTitles(error) \
                    -message $labelMessages(extNaN)  -type ok
            return
        }
        if {$extLimit<0} {
            set optionsTemp(exlevels) 0
        }
        if {$extLimit>3} {
            set optionsTemp(exlevels) 3
        }
    }

    set done 1

    return
}

###############################################################################
# ChooseOptions
#    Selects the different options.
#
# Parameter:
#    options: The array in which the options will be stored.
#    parent: Path of the window over which the dialog will appear
#
# Return
#    '1' if the user accepted the changes, '0' if he cancelled.
###############################################################################
proc ChooseOptions {optionsArray {parent .}} {
    variable optionsTemp
    variable window
    variable done

    upvar $optionsArray downOptions
    catch {unset optionsTemp}

    array set optionsTemp [array get downOptions]
    if {$optionsTemp(levels)==-1} {
        set optionsTemp(limitLevels) 0
        set optionsTemp(levels) 0
    } else {
        set optionsTemp(limitLevels) 1
    }

    ChooseOptionsWindow $parent
    UpdateSpinBox internal
    UpdateSpinBox external
    tkwait variable Opciones::done

    if {$done==1} {
        array set downOptions [array get optionsTemp]
        if {$optionsTemp(limitLevels)==0} {
            set downOptions(levels) -1
        }
    }

    destroy $window(win)

    return $done
}

}

